from setuptools import setup

setup(name='mrkv',
  version='0.0.1',
  description='Tiny Markov chain library',
  url='http://github.com/erdiaker/mrkv',
  author='Erdi Aker',
  author_email='erdiaker@gmail.com',
  license='MIT',
  py_modules=['mrkv'],
  install_requires=['future'],
  zip_safe=False)
